import { Data, animate, Override, Animatable } from "framer";

const data = Data({
  springDampingBoxPosition: Animatable(20),
  springTensionBoxPosition: Animatable(20),
  animateToXPosition: 360,

  springDampingBoxRotation: Animatable(0),
  springTensionBoxRotation: Animatable(0),
  animateToRotation: 180,

  springDampingBoxScale: Animatable(1),
  springTensionBoxScale: Animatable(1),
  animateToScale: 0.2,

  durationValue: 1,
  dampingRatioValue: 0.5,
  massValue: 1,
  velocityValue_Damping: 0,

  tensionValue: 250,
  frictionValue: 25,
  velocityValue_Tension: 3,

  toggleStartPosition: true,

  backgroundTextfieldDamping: Animatable("#2E3347"),
  backgroundTextfieldTension: Animatable("#2E3347")
});

// BOXES BEING ANIMATED
export const SpringDamping_MovingBox: Override = () => {
  return {
    left: data.springDampingBoxPosition
  };
};
export const SpringDamping_RotatingBox: Override = () => {
  return {
    rotation: data.springDampingBoxRotation
  };
};
export const SpringDamping_ScalingBox: Override = () => {
  return {
    scale: data.springDampingBoxScale
  };
};
export const SpringTension_MovingBox: Override = () => {
  return {
    left: data.springTensionBoxPosition
  };
};
export const SpringTension_RotatingBox: Override = () => {
  return {
    rotation: data.springTensionBoxRotation
  };
};
export const SpringTension_ScalingBox: Override = () => {
  return {
    scale: data.springTensionBoxScale
  };
};

// ANIMATIONS
const SpringDampingAnimation = (value, to) => {
  animate.spring(value, to, {
    duration: data.durationValue,
    dampingRatio: data.dampingRatioValue,
    mass: data.massValue,
    velocity: data.velocityValue_Damping
  });
};
const SpringTensionAnimation = (value, to) => {
  animate.spring(value, to, {
    tension: data.tensionValue,
    friction: data.frictionValue,
    velocity: data.velocityValue_Tension
    // tolerance: 0
  });
};

// SLIDERS FOR SPRING ANIMATION WITH DAMPING + THEIR READOUTS
export const DurationSlider: Override = () => {
  return {
    onValueChange(value) {
      data.durationValue = Math.round(value) / 10;
    }
  };
};
export const DurationReadout: Override = () => {
  return {
    text: data.durationValue
  };
};

export const DampingSlider: Override = () => {
  return {
    onValueChange(value) {
      data.dampingRatioValue = Math.round(value) / 100;
    }
  };
};
export const DampingReadout: Override = () => {
  return {
    text: data.dampingRatioValue
  };
};

export const MassSlider: Override = () => {
  return {
    onValueChange(value) {
      data.massValue = +value.toFixed(1);
    }
  };
};
export const MassReadout: Override = () => {
  return {
    text: data.massValue
  };
};

export const VelocitySlider_Damping: Override = () => {
  return {
    onValueChange(value) {
      data.velocityValue_Damping = Math.round(value);
    }
  };
};
export const VelocityReadout_Damping: Override = () => {
  return {
    text: data.velocityValue_Damping
  };
};

// SLIDERS FOR SPRING ANIMATION WITH TENSION + THEIR READOUTS
export const TensionSlider: Override = () => {
  return {
    onValueChange(value) {
      data.tensionValue = Math.round(value * 2.77777);
    }
  };
};
export const TensionReadout: Override = () => {
  return {
    text: data.tensionValue
  };
};

export const FrictionSlider: Override = () => {
  return {
    onValueChange(value) {
      data.frictionValue = Math.round(value);
    }
  };
};
export const FrictionReadout: Override = () => {
  return {
    text: data.frictionValue
  };
};

export const VelocitySlider_Tension: Override = () => {
  return {
    onValueChange(value) {
      data.velocityValue_Tension = Math.round(value);
    }
  };
};
export const VelocityReadout_Tension: Override = () => {
  return {
    text: data.velocityValue_Tension
  };
};

// TEXTFIELDS WITH EXAMPLE CODE
export const CodeForSpringDamping: Override = () => {
  return {
    text: `animate.spring(«valueToAnimate», «endValue», {
  duration: ${data.durationValue},
  dampingRatio: ${data.dampingRatioValue},
  mass: ${data.massValue},
  velocity: ${data.velocityValue_Damping}
});`
  };
};

export const CodeForSpringTension: Override = () => {
  return {
    text: `animate.spring(«valueToAnimate», «endValue», {
  tension: ${data.tensionValue},
  friction: ${data.frictionValue},
  velocity: ${data.velocityValue_Tension}
});`
  };
};

// BUTTON TO TRIGGER ANIMATIONS
export const StartAnimationsButton: Override = () => {
  return {
    onTap() {
      // Toggle start position
      const toggle = data.toggleStartPosition;
      if (toggle) {
        data.animateToXPosition = 340;
        data.animateToRotation = 180;
        data.animateToScale = 0.2;
      } else {
        data.animateToXPosition = 20;
        data.animateToRotation = 0;
        data.animateToScale = 1;
      }
      data.toggleStartPosition = !toggle;

      // Start animations
      SpringDampingAnimation(
        data.springDampingBoxPosition,
        data.animateToXPosition
      );
      SpringDampingAnimation(
        data.springDampingBoxRotation,
        data.animateToRotation
      );
      SpringDampingAnimation(data.springDampingBoxScale, data.animateToScale);

      SpringTensionAnimation(
        data.springTensionBoxPosition,
        data.animateToXPosition
      );
      SpringTensionAnimation(
        data.springTensionBoxRotation,
        data.animateToRotation
      );
      SpringTensionAnimation(data.springTensionBoxScale, data.animateToScale);
    }
  };
};

// FUNCTION FOR COPYING CODE TO CLIPBOARD
const copyTextToClipboard = function(text) {
  const textArea = document.createElement("textarea");

  // Place in top-left corner of screen regardless of scroll position.
  textArea.style.position = "fixed";
  textArea.style.top = "0px";
  textArea.style.left = "0px";

  //# Ensure it has a small width and height. Setting to 1px / 1em
  //# doesn't work as this gives a negative w/h on some browsers.
  textArea.style.width = "2em";
  textArea.style.height = "2em";

  //# We don't need padding, reducing the size if it does flash render.
  textArea.style.padding = "0px";

  // Clean up any borders.
  textArea.style.border = "none";
  textArea.style.outline = "none";
  textArea.style.boxShadow = "none";

  // Avoid flash of white box if rendered for any reason.
  textArea.style.background = "transparent";

  textArea.value = text;
  document.body.appendChild(textArea);
  textArea.select();

  const successful = document.execCommand("copy");

  return document.body.removeChild(textArea);
};

// TEXTFIELDS
export const TextfieldDamping: Override = () => {
  return {
    background: data.backgroundTextfieldDamping,
    onTap() {
      // Flash background
      data.backgroundTextfieldDamping.set("gray");
      animate.ease(data.backgroundTextfieldDamping, "#2E3347", {
        duration: 0.5
      });
      // Copy example code to Clipboard
      copyTextToClipboard(`animate.spring(«valueToAnimate», «endValue», {
  duration: ${data.durationValue},
  dampingRatio: ${data.dampingRatioValue},
  mass: ${data.massValue},
  velocity: ${data.velocityValue_Damping}
});`);
    }
  };
};
export const TextfieldTension: Override = () => {
  return {
    background: data.backgroundTextfieldTension,
    onTap() {
      // Flash background
      data.backgroundTextfieldTension.set("gray");
      animate.ease(data.backgroundTextfieldTension, "#2E3347", {
        duration: 0.5
      });
      // Copy example code to Clipboard
      copyTextToClipboard(`animate.spring(«valueToAnimate», «endValue», {
  tension: ${data.tensionValue},
  friction: ${data.frictionValue},
  velocity: ${data.velocityValue_Tension}
});`);
    }
  };
};

//// children test

export const childrenTest: Override = props => {
  return {
    children: React.Children.map(props.children), (child) => {
      console.log(child.props);
    }
    
  };
};
